{################################################################ OSIHEX.INC }
{ Purpose...: OSI include file: Send XASM65 HEX file
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 30.08.2022, 29.01.2023
{############################################################################}

{*************************************************************** definitions }
CONST m_iHEX_BAUDRATE:   INTEGER  = 300;         { baudrate for .HEX files   }
CONST m_iHEX_TXDLY_CHAR: INTEGER  = 0;           { tx delay between chars    }
CONST m_iHEX_TXDLY_LINE: INTEGER  = 1000;        { tx delay detween lines    }

{************************************************************ implementation }
{========================================================== Hex_GetAddress() }
{ Purpose...: Get address out of .HEX line
{ Input.....: i_stHex: HEX line to get address from
{ Result....: Address string
{============================================================================}
FUNCTION Hex_GetAddress(i_stLine: g_tStLine): g_tStHexAddr; BEGIN

  Hex_GetAddress := COPY(i_stLine, 4, 4);
END;
{====================================================== eoF Hex_GetAddress() }

{============================================================ Hex_SendLine() }
{ Purpose...: Send one HEX line to SIO
{ Input.....: i_stLine........: HEX line to send
{             i_pbNumBytesSent: Where to store number of bytes sent
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Hex_SendLine(i_stLine: g_tStLine; i_pbNumBytesSent: INTEGER): BYTE;

  VAR bSio:      BYTE;
  VAR bNumBytes: BYTE;
  VAR bCnt:      BYTE;
  VAR stAddress: g_tStHexAddr;
  VAR stData:    g_tStHexData;

BEGIN
  bCnt := 0;
  stAddress := Hex_GetAddress(i_stLine);
  bNumBytes := Lib_CnvHexToBin(COPY(i_stLine, 2, 2));
  WRITE(stAddress, ': ');
  bSio := LibSio_TxString('.' + stAddress, FALSE);

  WHILE ((bCnt < bNumBytes) AND (bSio = g_bLibSioOK)) DO BEGIN
    stData := COPY(i_stLine, ((bCnt * 2)+10), 2);
    WRITE(stData, ' ');
    bSio := LibSio_TxString(stData + g_cLF, FALSE);
    bCnt := bCnt + 1;
  END;

  WRITELN;
  MEM[i_pbNumBytesSent] := bCnt;
  Hex_SendLine := bSio;
END;
{======================================================== eoF Hex_SendLine() }

{============================================================ Hex_SendFile() }
{ Purpose...: Send .HEX file to SIO
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Hex_SendFile: BYTE;

  VAR bSio         : BYTE;
  VAR pstLine      : g_tpStLine;
  VAR stHexAddress : g_tStHexAddr;
  VAR iBytesSent   : INTEGER;
  VAR bBytesSent   : BYTE;
  VAR cKbd         : CHAR;
  VAR fStart       : BOOLEAN;
  VAR fDone        : BOOLEAN;

BEGIN
  bSio := g_bLibSioOK;
  iBytesSent := 0;

  {------------------------------- show usage hint and wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_TRANSMIT, g_stFile, ' (', File_GetBufLines, ' lines)');
  WRITE('In Monitor, type `L`.', g_stMSG_CONTINUE);
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
  END;

  {---------------------------------------------------------- initialize SIO }
  IF (bSio = g_bLibSioOK) THEN BEGIN

    LibSio_Init(TRUE, m_iHEX_BAUDRATE, 'N', 8, 1,
                m_iHEX_TXDLY_CHAR, m_iHEX_TXDLY_LINE);
  END;

  pstLine := PTR(1);
  fStart := TRUE;
  fDone := FALSE;

  {------------------------------------------------- parse and send the file }
  WHILE ((pstLine <> NIL) AND (bSio = g_bLibSioOK)) DO BEGIN
    pstLine := File_GetString(-1, fStart, NIL);
    fStart := FALSE;

    IF (pstLine <> NIL) THEN BEGIN

      {----------------- check for end record, it contains the start address }
      IF (Lib_CnvHexToBin(COPY(pstLine^, 2, 2)) = 0) THEN BEGIN
        stHexAddress := Hex_GetAddress(pstLine^);
        bSio := LibSio_TxString('.' + stHexAddress + 'G' + g_stCRLF, FALSE);
        fDone := TRUE
      END ELSE BEGIN
        bSio := Hex_SendLine(pstLine^, ADDR(bBytesSent));
        iBytesSent := iBytesSent + bBytesSent;
      END;
    END;
  END;

  WRITELN;

  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITE('Cancelled. ');
  END;

  IF (fDone = TRUE) THEN BEGIN
    WRITELN('Start Address: ', stHexAddress);
  END;

  WRITELN('Bytes sent: ', iBytesSent);
  WRITELN('Press <SPACE> on OSI to end LOAD');

  LibSio_Init(FALSE, 0, 'N', 0, 0, 0, 0);
  Hex_SendFile := bSio;
END;
{======================================================== eoP Hex_SendFile() }

{############################################################ EOF OSIHEX.INC }