{############################################################### OSIMENU.INC }
{ Purpose...: OSI menu, menu and file selection
{ DependsOn.: LIB.INC, OSIFILE.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 03.09.2022, 29.01.2023
{############################################################################}

{----- line number where selection prompt is printed (used for file listing) }
CONST m_bPROMPT_LINE = 13;

{----------------------------------------------- length of menu border lines }
CONST m_bBORDER_LENGTH = 60;

{************************************************************ implementation }
{========================================================= Menu_SelectFile() }
{ Purpose...: Select a file
{ Input.....: i_bOperation: m_bOPERATION_XXX
{ Result....: TRUE= File entered/selected, FALSE= aborted
{============================================================================}
FUNCTION Menu_SelectFile(i_bOperation: BYTE): BOOLEAN;

  CONST bMODE_TRANSMIT = 1;
  CONST bMODE_RECEIVE  = 2;
  CONST bMODE_VIEW     = 3;
  CONST bMODE_PRINT    = 4;
  
  VAR stFileExt: g_tStFile;
  VAR bCnt:      BYTE;
  VAR bMode:     BYTE;

BEGIN
  bMode := bMODE_RECEIVE;
  stFileExt := '';
  WRITE(g_cCR, 'Reading directory...');

  CASE i_bOperation OF
    g_bOPERATION_TRANSMIT_HEX: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.HEX';
                               END;
    g_bOPERATION_RECEIVE_LOD:  BEGIN
                                 stFileExt := '.LOD';
                               END;
    g_bOPERATION_TRANSMIT_LOD: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.LOD';
                               END;
    g_bOPERATION_RECEIVE_ASM:  BEGIN
                                 stFileExt := '.ASM';
                               END;
    g_bOPERATION_TRANSMIT_ASM: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.ASM';
                               END;
    g_bOPERATION_RECEIVE_BAS:  BEGIN
                                 stFileExt := '.BAS';
                               END;
    g_bOPERATION_TRANSMIT_BAS: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.BAS';
                               END;
    g_bOPERATION_VIEW_TXT:     BEGIN
                                 bMode := bMODE_VIEW;
                                 stFileExt := '.???';
                               END;
    g_bOPERATION_PRINT_TXT:    BEGIN
                                 bMode := bMODE_PRINT;
                                 stFileExt := '.???';
                               END;  
  END;

  {-------------------------------- display directory of selected file types }
  WRITELN; WRITELN;
  IF (stFileExt = '.HEX') THEN WRITE('Xasm65 HEX');
  IF (stFileExt = '.LOD') THEN WRITE('Monitor');
  IF (stFileExt = '.ASM') THEN WRITE('AsmEdit');
  IF (stFileExt = '.BAS') THEN WRITE('Basic');
  IF (stFileExt = '.???') THEN WRITE('Text');
  WRITELN(' files on disk');
  Lib_PrintDirectory('????????' + COPY(stFileExt,2,4));

  {------------------------------------------------------------ get filename }
  GOTOXY(1, m_bPROMPT_LINE);
  CLREOL;

  IF (stFileExt <> '.???') THEN BEGIN
    WRITE(g_cCR, 'File (', stFileExt, ') to ');
  END ELSE BEGIN
    WRITE(g_cCR, 'File to ');
  END;

  CASE bMode OF
    bMODE_TRANSMIT: WRITE('transmit ');
    bMODE_RECEIVE:  WRITE('receive ');
    bMODE_VIEW:     WRITE('view ');
    bMODE_PRINT:    WRITE('print ');
  END;

  g_stFile := '';
  READ(g_stFile);
  WRITE(g_cCR);

  {-------------------------------- clear area occupied by directory listing }
  FOR bCnt := m_bPROMPT_LINE TO 23 DO BEGIN
    GOTOXY(1, bCnt);
    CLREOL;
  END;

  GOTOXY(1, m_bPROMPT_LINE);

  IF (stFileExt <> '.???') THEN BEGIN
    IF (LENGTH(g_stFile) >8) THEN BEGIN
      g_stFile := COPY(g_stFile, 1, 8);
    END;
  END;

  {-------------------------------------------------------- open/create file }
  IF (LENGTH(g_stFile) <> 0) THEN BEGIN

    IF (stFileExt <> '.???') THEN BEGIN
      g_stFile := Lib_CnvUpCase(g_stFile + stFileExt);
    END ELSE BEGIN
      g_stFile := Lib_CnvUpCase(g_stFile);
    END;

    IF (bMode <> bMODE_PRINT) THEN BEGIN
      IF (File_OpenFile(i_bOperation) = FALSE) THEN BEGIN
        g_stFile := '';
      END;
    END;
  END;

  Menu_SelectFile := (LENGTH(g_stFile) <> 0);
END;
{===================================================== eoF Menu_SelectFile() }

{========================================================== Menu_PrintFile() }
{ Purpose...: Print a file
{============================================================================}
PROCEDURE Menu_PrintFile;
  VAR bCnt : BYTE;
  
BEGIN  
  WRITE('Printing ' + g_stFile);
  LibPrt_PrintFile(g_stFile, 1, m_bPROMPT_LINE + 1);
  
  {--------------------------------------- clear area occupied by message(s) }
  FOR bCnt := m_bPROMPT_LINE TO (m_bPROMPT_LINE + 2) DO BEGIN
    GOTOXY(1, bCnt);
    CLREOL;
  END;
  
  GOTOXY(1, m_bPROMPT_LINE);  
END;
{====================================================== eoP Menu_PrintFile() }

{========================================================= Menu_PrintSetup() }
{ Purpose...: Select a print mode, send command to printer
{============================================================================}
PROCEDURE Menu_PrintSetup;

  TYPE tstMode     = STRING[10];

  VAR cKbd         : CHAR;
  VAR fDone        : BOOLEAN;
  VAR fDummy       : BOOLEAN;
  VAR stMode       : tstMode;
  VAR iLeftMargin  : INTEGER;
  VAR piLeftMargin : ^INTEGER;
  VAR bPageLength  : BYTE;
  VAR bPageLines   : BYTE;
  VAR iPageNum     : INTEGER;

BEGIN
  fDone := FALSE;
  stMode := '';
  piLeftMargin := PTR(ADDR(iLeftMargin));

  {---------------------------------------------------------------- get mode }
  WHILE (fDone = FALSE) DO BEGIN
    GOTOXY(1, m_bPROMPT_LINE);
    CLREOL;
    WRITE('Init  Pica  Elite  Cond  eMph  mArgin  Form  pageNbr ');
    cKbd := UPCASE(Lib_GetKey('IiPpCcEeMmAaFfNn' + g_cESC + g_cCR, TRUE));
    GOTOXY(1, m_bPROMPT_LINE);
    CLREOL;

    LibPrt_Setup(1, m_bPROMPT_LINE, FALSE, -1);

    CASE cKbd OF
      'I'  : stMode := g_cLibPrtMODE_INIT;
      'P'  : stMode := g_cLibPrtMODE_PICA;
      'E'  : stMode := g_cLibPrtMODE_ELITE;
      'C'  : stMode := g_cLibPrtMODE_COND;
      'M'  : stMode := g_cLibPrtMODE_EMPH;
      'A'  : stMode := g_cLibPrtMODE_LMARGIN;
    END;

    fDone := ((cKbd = g_cESC) OR (cKbd = g_cCR));

    IF (fDone = FALSE) THEN BEGIN

      {------------------------------------------- enter left printer margin }
      IF (cKbd = 'A') THEN BEGIN
        WRITE('Left margin: ');

        IF (Lib_GetNum(piLeftMargin, NIL, 3, FALSE) = TRUE) THEN BEGIN
          stMode := stMode + CHR(iLeftMargin);
        END ELSE BEGIN
          stMode := '';
        END;

        GOTOXY(1, m_bPROMPT_LINE);
        CLREOL;
      END;

      {-------------------------------------------------- print page numbers }
      IF (cKbd = 'N') THEN BEGIN
        WRITE('Print page numbers (Y/N): ');
        cKbd := UPCASE(Lib_GetKey('YyNn' + g_cESC + g_cCR, TRUE));
        iPageNum := 0;
        IF (cKbd = 'Y') THEN iPageNum := 1;
        LibPrt_Setup(-1, -1, FALSE, iPageNum);
        stMode := '';

        GOTOXY(1, m_bPROMPT_LINE);
        CLREOL;
      END;

      {------------------------------------------------- select paper format }
      IF (cKbd = 'F') THEN BEGIN
        WRITE('<A>4  <B>5  <C>ontinuous ');
        cKbd := UPCASE(Lib_GetKey('AaBbCc' + g_cESC + g_cCR, TRUE));
        bPageLength := 0;
        bPageLines := 0;

        IF (cKbd = 'A') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_A4;
          bPageLines := g_bLibPrtPAGE_LINES_A4;
        END;

        IF (cKbd = 'B') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_B5;
          bPageLines := g_bLibPrtPAGE_LINES_B5;
        END;

        IF (cKbd = 'C') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_CONT OR $80;
          bPageLines := g_bLibPrtPAGE_LINES_CONT;
        END;

        IF (bPageLength <> 0) THEN BEGIN
          LibPrt_SetForm(bPageLength, bPageLines);
        END;
      END;

      IF (LENGTH(stMode) <> 0) THEN BEGIN
        fDummy := LibPrt_Print(CHR(0), g_cESC + stMode);
      END;
    END;
  END;
END;
{===================================================== eoP Menu_PrintSetup() }

{============================================================= Menu_Border() }
{ Purpose...: Draw menu borders. Implemented to save space (hopefully)
{============================================================================}
PROCEDURE Menu_Border;

  VAR bCnt : BYTE;

BEGIN
  FOR bCnt := 0 TO m_bBORDER_LENGTH DO BEGIN
    WRITE('=');
  END;

  WRITELN;
END;
{=========================================================== eoP Menu_Border }

{==================================================================== Menu() }
{ Purpose...: Display menu and call requested handlers. Returns on Exit
{             selection only
{============================================================================}
PROCEDURE Menu; 

  VAR cKbd       : CHAR;
  VAR bOperation : BYTE;
  VAR bOpResult  : BYTE;
  VAR fDrawMenu  : BOOLEAN;
  VAR fExit      : BOOLEAN;
  VAR stFileExt  : g_tStFile;

BEGIN
  fExit := FALSE;
  fDrawMenu := TRUE;

  WHILE (fExit = FALSE) DO BEGIN

    IF (fDrawMenu = TRUE) THEN BEGIN
      fDrawMenu := FALSE;
      CLRSCR;
      Menu_Border;
      WRITELN(g_stPROGRAM, ' V', g_stVERSION);
      Menu_Border;

      WRITELN('-1- Transmit HEX file');
      WRITELN('-2- Transmit Monitor file         -3- Receive Monitor file');    
      WRITELN('-4- Transmit Assembler file       -5- Receive Assembler file');
      WRITELN('-6- Transmit BASIC file           -7- Receive BASIC File');
      WRITELN('-T- Terminal');
      WRITELN('-V- View text file                -U- View Usage notes');
      WRITELN('-P- Print text file               -S- Setup Printer');
      WRITELN('-X- Exit');
            
      Menu_Border;
    END;

    {--------------------------------------------------- read menu selection }
    WRITE(g_cCR);
    CLREOL;
    WRITE('Selection ');
    cKbd := UPCASE(Lib_GetKey('1234567TtVvPpUuSsXx', TRUE));
    bOperation := 0;
        
    CASE cKbd OF
      '1': bOperation := g_bOPERATION_TRANSMIT_HEX;
      '2': bOperation := g_bOPERATION_TRANSMIT_LOD;
      '3': bOperation := g_bOPERATION_RECEIVE_LOD;
      '4': bOperation := g_bOPERATION_TRANSMIT_ASM;
      '5': bOperation := g_bOPERATION_RECEIVE_ASM;
      '6': bOperation := g_bOPERATION_TRANSMIT_BAS;
      '7': bOperation := g_bOPERATION_RECEIVE_BAS;
      'T': bOperation := g_bOPERATION_TERMINAL;
      'V': bOperation := g_bOPERATION_VIEW_TXT;
      'P': bOperation := g_bOPERATION_PRINT_TXT;
      'U': bOperation := g_bOPERATION_USAGE_NOTES;
      'S': bOperation := g_bOPERATION_PRINT_SETUP;
      'X': fExit := TRUE;
    END;

    {---------------------------------------------- select file to work with }
    IF ((bOperation AND g_bOPMASK_SELECT_FILE) <> 0) THEN BEGIN
      IF (Menu_SelectFile(bOperation) = FALSE) THEN BEGIN
        bOperation := 0;
      END;
    END;

    {------------------------------------------------ call requested handler }
    bOpResult := 0;
    
    CASE bOperation OF

      {------------------------------------------------------- send hex file }
      g_bOPERATION_TRANSMIT_HEX: bOpResult := Hex_SendFile;

      {---------------------------------------------------- receive lod file }
      g_bOPERATION_RECEIVE_LOD:  bOpResult := Lod_ReceiveFile;

      {------------------------------------------------------- send lod file }
      g_bOPERATION_TRANSMIT_LOD: bOpResult := Lod_SendFile;

      {---------------------------------------------------- receive asm file }
      g_bOPERATION_RECEIVE_ASM:  bOpResult := Asm_ReceiveFile;

      {------------------------------------------------------- send asm file }
      g_bOPERATION_TRANSMIT_ASM: bOpResult := Asm_SendFile;

      {-------------------------------------------------- receive basic file }
      g_bOPERATION_RECEIVE_BAS:  bOpResult := Bas_ReceiveFile;

      {----------------------------------------------------- send basic file }
      g_bOPERATION_TRANSMIT_BAS: bOpResult := Bas_SendFile;

      {------------------------------------------------------- terminal mode }
      g_bOPERATION_TERMINAL:     Term;

      {------------------------------------------------------ view text file }
      g_bOPERATION_VIEW_TXT:     View;

      {---------------------------------------------------- view usage notes }
      g_bOPERATION_USAGE_NOTES:
        BEGIN
          g_stFile := g_stFILE_USAGE;
             
          IF (File_OpenFile(g_bOPERATION_VIEW_TXT) = FALSE) THEN BEGIN
            g_stFile := '';
            bOperation := 0;
          END
          ELSE BEGIN
            View;
          END;  
        END;

      {--------------------------------------------------- select print mode }
      g_bOPERATION_PRINT_SETUP:  Menu_PrintSetup;
       
      {----------------------------------------------------- print text file }
      g_bOPERATION_PRINT_TXT:    Menu_PrintFile;
    END;
    
    {---------------------------------------------- show SIO timeout message }
    IF (bOpResult = g_bLibSioTIMEOUT) THEN BEGIN
      fDrawMenu := TRUE;
      WRITELN;
      WRITELN('SIO timed out');
      File_CloseFile;
    END;

    {-------------------- continuation prompt except for terminal and viewer }
    IF ((bOperation AND g_bOPMASK_CONT_PROMPT) <> 0) THEN BEGIN
      fDrawMenu := TRUE;
      File_CloseFile;
      WRITELN;
      WRITE('Any key to return to menu ');
      WHILE (NOT KEYPRESSED) DO BEGIN END;
    END;

    {-------------------------------------------------- force redraw of menu }
    IF ((bOperation AND g_bOPMASK_MENU) <> 0) THEN BEGIN
      fDrawMenu := TRUE;
    END;  
  END;

  CLRSCR;
END;
{================================================================ eoF Menu() }

{########################################################### EOF OSIMENU.INC }