{############################################################## KIM1VIEW.INC }
{ Purpose...: KIM1 include file: View text file
{ DependsOn.: LIB.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 25.01.2023
{############################################################################}

{*************************************************************** definitions }
CONST m_bMSG_ROW   = 1;
CONST m_bMSG_COL   = 24;
CONST m_bSCR_LINES = 22;

{************************************************************ implementation }
{============================================================= View_Prompt() }
{ Purpose...: Print prompt message 
{ Input.....: i_stMsg: Prompt string
{=========================================================================== }
PROCEDURE View_Prompt(i_stMsg: g_tst);

BEGIN
  GOTOXY(m_bMSG_ROW, m_bMSG_COL);
  CLREOL;
  WRITE(i_stMsg);
END;
{========================================================= eoP View_Prompt() }

{============================================================= View_GetCmd() }
{ Purpose...: Print command prompt on bottom line and get user response
{ Input.....: i_fMore.: TRUE= there is more to come, <F>orward shown
{             i_fFirst: TRUE= first screen shown, <B>ackward not shown
{ Result....: Entered character
{============================================================================}
FUNCTION View_GetCmd(i_fFirst: BOOLEAN; i_fMore: BOOLEAN): CHAR;

  VAR stKeys : STRING[9];
  VAR stMsg  : g_tst;

BEGIN
  stKeys := g_cESC;
  stMsg := g_stFile + ': ';

  IF (i_fMore = TRUE) THEN BEGIN
    stMsg := stMsg + '<F>wd | ';
    stKeys := stKeys + 'Ff';
  END;

  IF (i_fFirst = FALSE) THEN BEGIN
    stMsg := stMsg + '<B>ack | ';
    stKeys := stKeys + 'Bb';
  END;

  stMsg := stMsg + '<P>rint screen | <esc> exit ';
  stKeys := stKeys + 'Pp';

  View_Prompt(stMsg);

  View_GetCmd := UPCASE(Lib_GetKey(stKeys, TRUE));
END;
{========================================================= eoF View_GetCmd() }

{==================================================================== View() }
{ Purpose...: View text file
{============================================================================}
PROCEDURE View;

  VAR pstLine       : g_tpstLine;
  VAR stLine        : g_tstLine;
  VAR pfEof         : ^BOOLEAN;
  VAR fEof          : BOOLEAN;
  VAR iBufLine      : INTEGER;
  VAR iFirstLine    : INTEGER;
  VAR iFirstLineBak : INTEGER;
  VAR iScrLines     : INTEGER;
  VAR iNumLines     : INTEGER;
  VAR fPrintScreen  : BOOLEAN;
  VAR fPrtTimeout   : BOOLEAN;
  VAR fStart        : BOOLEAN;
  VAR fExit         : BOOLEAN;
  VAR cKbd          : CHAR;
  VAR bNdx          : BYTE;
  VAR bLen          : BYTE;
  VAR bTabSize      : BYTE;

BEGIN
  pstLine := NIL;
  pfEof := PTR(ADDR(fEof));
  fEof := FALSE;
  iBufLine := 0;
  iFirstLine := 0;
  iFirstLineBak := 0;
  iScrLines := 0;
  fPrintScreen := FALSE;
  fPrtTimeout := FALSE;
  fStart := TRUE;
  fExit := FALSE;
  cKbd := CHR(0);
  iNumLines := File_GetBufLines;

  {-------------------------- set top-of-form in case screen prints are done }
  LibPrt_Setup(m_bMSG_ROW, m_bMSG_COL, TRUE, -1);

  {------------------------------------------------ loop until exit selected }
  WHILE (fExit = FALSE) DO BEGIN
    fPrtTimeout := FALSE;
    IF (fPrintScreen = FALSE) THEN CLRSCR;
    pstLine := PTR(1);
    iBufLine := iFirstLine;
    iScrLines := 0;
    fEof := FALSE;

    {---------------------------------------------------- process one screen }
    WHILE ((pstLine <> NIL) AND (iScrLines < m_bSCR_LINES)) DO BEGIN
      pstLine := File_GetString(iBufLine, fStart, pfEof);
      fStart := FALSE;
      iBufLine := -1;
      fEof := pfEof^;

      {----------------------------------------- continue if not end of file }
      IF (pstLine <> NIL) THEN BEGIN

        {----------------------------------------- print or display one line }
        IF (fPrintScreen = TRUE) THEN BEGIN
          fPrtTimeout := NOT LibPrt_Print(CHR(0), pstLine^ + g_stCRLF);
        END;

        {------------------------------------------- expand tabs to 8 spaces }
        IF (fPrintScreen = FALSE) THEN BEGIN

          IF (POS(g_cTAB, pstLine^) <> 0) THEN BEGIN
            bNdx := 0;
            bLen := LENGTH(pstLine^);
            stLine := '';

            WHILE (bNdx < bLen) DO BEGIN
              bNdx := bNdx + 1;

              IF (pstLine^[bNdx] = g_cTAB) THEN BEGIN
                bTabSize := 8 - (LENGTH(stLine) MOD 8);
                stLine := stLine + COPY('        ', 1, bTabSize);
              END ELSE BEGIN
                stLine := stLine + pstLine^[bNdx];
              END;
            END;
          END ELSE BEGIN
            stLine := pstLine^;
          END;

          WRITELN(COPY(stLine, 1, 79));
        END;

        iScrLines := iScrLines + 1;

        {------------------------------- allow printing to abort using <esc> }
        IF (fPrintScreen = TRUE) THEN BEGIN
          IF (Lib_GetKey(g_cESC, FALSE) = g_cESC) THEN BEGIN
            iScrLines := m_bSCR_LINES;
          END;
        END;

        IF (fPrtTimeout = TRUE) THEN BEGIN
          iScrLines := m_bSCR_LINES;
        END;
      END;
    END;

    {------------------------------------------------ cleanup after printing }
    IF (fPrintScreen = TRUE) THEN BEGIN
      fPrintScreen := FALSE;
      iFirstLine := iFirstLineBak;
    END;

    {------------------------------------------------ read command from user }
    cKbd := View_GetCmd((iFirstLine = 0), (fEof = FALSE));

    {--------------------------------------------------- display next screen }
    IF (cKbd = 'F') THEN BEGIN
      iFirstLine := iFirstLine + m_bSCR_LINES;
    END;

    {----------------------------------------------- display previous screen }
    IF (cKbd = 'B') THEN BEGIN
      iFirstLine := iFirstLine - m_bSCR_LINES;
      fStart := TRUE;
    END;

    {------------------------------------ print actual screen, backup values }
    IF (cKbd = 'P') THEN BEGIN
      View_Prompt(g_stFile + ': Printing screen' + g_stMSG_CANCEL);
      iFirstLineBak := iFirstLine;
      fPrintScreen := TRUE;
      fStart := TRUE;
    END;

    {-------------------------------------- exit, free buffer and close file }
    IF (cKbd = g_cESC) THEN BEGIN
      File_CloseFile;
      CLRSCR;
      fExit := TRUE;
    END;
  END;
END;
{================================================================ eoP View() }

{########################################################## EOF KIM1VIEW.INC }